module Evolution
  use Consts
  implicit none

contains
  
  subroutine Res_S(alphas_nu, alphas_mu, gamma, beta, order, S)
    implicit none
    complex(8), intent(in) :: alphas_nu, alphas_mu
    real(8), intent(in) :: gamma(0:3), beta(0:3)
    integer, intent(in) :: order
    complex(8), intent(out) :: S(0:2)
    complex(8) :: r

    r = alphas_mu / alphas_nu
    S = 0d0

    S(0) =  4d0*pi * (1d0-1d0/r-log(r)) / alphas_nu &
         + (gamma(1)/gamma(0) - beta(1)/beta(0)) * (1d0-r+log(r)) &
         + beta(1)/2d0/beta(0) * log(r)**2
    if (order > 0) then
       S(1) = alphas_nu / (4d0*pi) * ( &
            (beta(1)*gamma(1)/beta(0)/gamma(0) - beta(2)/beta(0)) * (1d0-r+r*log(r)) &
            + (beta(1)**2/beta(0)**2 - beta(2)/beta(0)) * (1d0-r) * log(r) &
            - (beta(1)**2/beta(0)**2 - beta(2)/beta(0) - beta(1)*gamma(1)/beta(0)/gamma(0) &
            + gamma(2)/gamma(0)) * (1d0-r)**2 / 2d0 )
    endif
    if (order > 1) then
       S(2) = alphas_nu**2 / (4d0*pi)**2 * ( &
            ( beta(1)*beta(2)/beta(0)**2 - beta(1)**3/2d0/beta(0)**3 - beta(3)/2d0/beta(0) &
            + beta(1)/beta(0) * (gamma(2)/gamma(0) - beta(2)/beta(0) &
            + beta(1)**2/beta(0)**2 - beta(1)*gamma(1)/beta(0)/gamma(0)) * r**2/2d0 ) &
            * log(r) &
            + ( gamma(3)/gamma(0) - beta(3)/beta(0) + 2d0*beta(1)*beta(2)/beta(0)**2 &
            + beta(1)**2/beta(0)**2 * (gamma(1)/gamma(0) - beta(1)/beta(0)) &
            - beta(2)*gamma(1)/beta(0)/gamma(0) - beta(1)*gamma(2)/beta(0)/gamma(0) ) &
            * (1d0-r)**3/3d0 &
            + ( 3d0*beta(3)/4d0/beta(0) - gamma(3)/2d0/gamma(0) + beta(1)**3/beta(0)**3 &
            - 3d0*beta(1)**2*gamma(1)/4d0/beta(0)**2/gamma(0) &
            + beta(2)*gamma(1)/beta(0)/gamma(0) &
            + beta(1)*gamma(2)/4d0/beta(0)/gamma(0) - 7d0*beta(1)*beta(2)/4d0/beta(0)**2 ) &
            * (1d0-r)**2 &
            + (beta(1)*beta(2)/beta(0)**2 - beta(3)/beta(0) &
            - beta(1)**2*gamma(1)/beta(0)**2/gamma(0) &
            + beta(1)*gamma(2)/beta(0)/gamma(0)) * (1d0-r)/2d0 )
    endif
    S = S * gamma(0) / (2d0*beta(0))**2
  end subroutine Res_S

  subroutine Res_aG(alphas_nu, alphas_mu, gamma, beta, order, a)
    implicit none
    complex(8), intent(in) :: alphas_nu, alphas_mu
    real(8), intent(in) :: gamma(0:3), beta(0:3)
    integer, intent(in) :: order
    complex(8), intent(out) :: a(0:2)

    a = 0d0
    a(0) = gamma(0) * log(alphas_mu/alphas_nu) &
         +  (gamma(1) - gamma(0)*beta(1)/beta(0)) * (alphas_mu - alphas_nu) / (4d0*pi)
    if (order > 0) then
       a(1) = (gamma(2) - gamma(0)*beta(2)/beta(0) - beta(1)/beta(0) * (gamma(1) &
            - gamma(0)*beta(1)/beta(0))) * (alphas_mu**2 - alphas_nu**2) / (32d0*pi**2)
    endif
    if (order > 1) then
       a(2) = (gamma(3) - gamma(2)*beta(1)/beta(0) &
            + (beta(1)**2/beta(0)**2 - beta(2)/beta(0)) * gamma(1) &
            + (-beta(1)**3/beta(0)**3 + 2d0*beta(2)*beta(1)/beta(0)**2 &
            - beta(3)/beta(0)) * gamma(0)) * (alphas_mu**3 - alphas_nu**3) / 3d0 / (4d0*pi)**3
    endif

    a = a / (2d0*beta(0))
  end subroutine Res_aG

  subroutine Res_a(alphas_nu, alphas_mu, gamma, beta, order, a)
    implicit none
    complex(8), intent(in) :: alphas_nu, alphas_mu
    real(8), intent(in) :: gamma(0:2), beta(0:3)
    integer, intent(in) :: order
    complex(8), intent(out) :: a(0:2)

    a = 0d0
    a(0) = gamma(0) * log(alphas_mu/alphas_nu)
    if (order > 0) then
       a(1) = (gamma(1) - gamma(0)*beta(1)/beta(0)) * (alphas_mu - alphas_nu) / (4d0*pi)
    endif
    if (order > 1) then
       a(2) = (gamma(2) - gamma(0)*beta(2)/beta(0) - beta(1)/beta(0) * (gamma(1) &
            - gamma(0)*beta(1)/beta(0))) * (alphas_mu**2 - alphas_nu**2) / (32d0*pi**2)
    endif

    a = a / (2d0*beta(0))
  end subroutine Res_a

end module Evolution
