module HiggsFixed
  use Consts
  use HiggsVar
  use Higgs
  implicit none
  
contains

  subroutine Higgs_Nonsingular_CS_Integrand(ndim, y, ncomp, r)
    implicit none
    integer, intent(in) :: ndim, ncomp
    real(8), intent(in) :: y(ndim)
    real(8), intent(out) :: r(ncomp)
    real(8) :: x1, x2, z, rNN, l1z
    real(8) :: zeta2, zeta3, LfH, lz, l1mz, l1pz
    real(8) ::li2z, li2mz, li2z2, li3z, li3mz, li3z2, s12z, s12mz, s12z2
    common /babis/ zeta2, zeta3, LfH, lz, l1mz, l1pz, li2z, li2mz, li2z2, &
         li3z, li3mz, li3z2, s12z, s12mz, s12z2
    real(8), external :: my_xggrNN, my_xqgNN, my_xqqbNN, my_xqqNN, my_xqqprime
    real(8) :: lumigg, lumigq, lumiqqbar, lumiqq, lumiqqprime 
    real(8) :: pdf1(-6:6), pdf2(-6:6)
    integer :: i, j

    zeta2 = z2
    zeta3 = z3

    z = tau + (1d0-tau)*y(1)
    x1 = tau/z + (1d0-tau/z)*y(2)
    x2 = tau/z/x1

    call evolvePDF(x1, mu_f, pdf1)
    call evolvePDF(x2, mu_f, pdf2)
    
    lumigg = pdf1(0)*pdf2(0)

    lumigq = 0d0
    do i = 1, 5
       lumigq = lumigq + pdf1(0)*(pdf2(i)+pdf2(-i)) + (pdf1(i)+pdf1(-i))*pdf2(0)
    end do

    lumiqqbar = 0d0
    do i = 1, 5
       lumiqqbar = lumiqqbar + pdf1(i)*pdf2(hadron2*(-i)) + pdf1(-i)*pdf2(hadron2*i)
    end do

    lumiqq = 0d0
    do i = 1, 5
       lumiqq = lumiqq + pdf1(i)*pdf2(hadron2*i) + pdf1(-i)*pdf2(hadron2*(-i))
    end do

    lumiqqprime = 0d0
    do i = -5, 5
       do j = -5, 5
          if (i == j .or. i == -j .or. i == 0 .or. j == 0) cycle
          lumiqqprime = lumiqqprime + pdf1(i)*pdf2(hadron2*j)
       end do
    end do

    l1z = log(mH**2*(1-z)**2/mu_f**2/z)
    LfH = 2d0*log(mu_f/mH)
    lz = log(z)
    l1mz = log(1d0-z)
    l1pz = log(1d0+z)
    li2z = ddilog(z)
    li2mz = ddilog(-z)
    li2z2 = ddilog(z**2)
    li3z = wgplg(2,1,z)
    li3mz = wgplg(2,1,-z)
    li3z2 = wgplg(2,1,z**2)
    s12z = wgplg(1,2,z)
    s12mz = wgplg(1,2,-z)
    s12z2 = wgplg(1,2,z**2)

    r = lumigg * (6d0*(1d0/z-2d0+z-z**2)*l1z - (11d0/2d0)*(1d0-z)**3/z)
    r = r + lumigq * ((2d0/3d0)*((1d0+(1d0-z)**2)/z)*l1z + 2d0*z/3d0 - (1d0-z)**2/z)
    r = r + lumiqqbar * ((32d0/27d0)*(1d0-z)**3/z)
    r = r * (alphas_muf/pi)

    if (higgs_order > 1) then
       rNN = lumigg * my_xggrNN(z) + lumigq * my_xqgNN(z) + lumiqqbar * my_xqqbNN(z) &
            + lumiqq * my_xqqNN(z) + lumiqqprime * my_xqqprime(z)
       r = r + rNN * (alphas_muf/pi)**2
    end if

    r = r * prefactor * alphas_muf**2 * (1d0-tau) * (1d0-tau/z) / x1 / tau
  end subroutine Higgs_Nonsingular_CS_Integrand
  
  function Higgs_Nonsingular_CS() result(cs)
    implicit none
    real(8) :: cs
    real(8) :: r(1)

    call InitHiggs(.false.)

    cs = 0d0
    if (higgs_order > 0) then
       call Cuba(Cuba_CUHRE, 2, 1, Higgs_Nonsingular_CS_Integrand, r, iERR)
       cs = r(1)
    end if
  end function Higgs_Nonsingular_CS

end module HiggsFixed
