program RunHiggs
  use Higgs
  use HiggsFixed
  use HiggsEW
  implicit none

  character*100 :: pdfset, pdfset_asmz(0:4)
  real(8) :: asmz_asmz(0:4)
  integer :: nmem, n_asmz
  logical :: with_asmz_error

  integer :: fd_in = 10, op, i

  open(fd_in, file='RunHiggs.in', status='old')

  read(fd_in, '(A50)'), pdfset
  read(fd_in, *), alphas_mZ
  read(fd_in, *), alphas_order
  read(fd_in, *), Ecms
  read(fd_in, *), hadron2
  read(fd_in, *), mH
  read(fd_in, *), muf0
  read(fd_in, *), higgs_order
  read(fd_in, *), sub_order
  read(fd_in, *), mt
  read(fd_in, *), mbmb
  read(fd_in, *), GFew
  read(fd_in, *), with_asmz_error
  if (with_asmz_error) then
     read(fd_in, *), n_asmz
     do i = 1, n_asmz
        read(fd_in, *), pdfset_asmz(i)
        read(fd_in, *), asmz_asmz(i)
     end do
  end if
  close(fd_in)
  mut0 = -1
  muh0 = -1
  mus0 = -1

  call InitQCD
  call InitEW

  call InitPDFSetByName(pdfset)
  call numberPDF(nmem)
  call InitPDF(0)

  call SetLHAPARM('SILENT')

  call Higgs_UpdateCoeff

  call Run_Single

contains

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  subroutine Run_Single
    implicit none
    real(8) :: r_fixed(-1:1), r_onlyth(-1:1), r_onlyipi(-1:1), r_ipith(-1:1)
    real(8) :: rp_fixed(-1:1), rp_onlyth(-1:1), rp_onlyipi(-1:1), rp_ipith(-1:1)
    character*100 :: format_string = '(A30,2X,F8.3,SP,2X,F8.3,2X,F8.3)'

    print '(A8,1X,F6.0,1X,A3)', "sqrtS =", Ecms, "GeV"
    print '(A8,1X,F6.1,1X,A3)', "m_H =", mH, "GeV"
    print *, ""

    call Higgs_scale_error(.false., r_fixed, r_onlyth, r_onlyipi, r_ipith)
    print *, "Cross sections with scale uncertainties (pb)"
    print format_string, "Fixed order:", r_fixed(0), r_fixed(1), r_fixed(-1)
    print format_string, "Fixed order + EW:", &
         r_fixed(0)*(1d0+delta_EW), r_fixed(1)*(1d0+delta_EW), r_fixed(-1)*(1d0+delta_EW)
    print format_string, "Only threshold resummed:", r_onlyth(0), r_onlyth(1), r_onlyth(-1)
    print format_string, "Only pi^2 resummed:", r_onlyipi(0), r_onlyipi(1), r_onlyipi(-1)
    print format_string, "Threshold+pi^2 resummed:", r_ipith(0), r_ipith(1), r_ipith(-1)
    print format_string, "Threshold+pi^2 resummed + EW:", &
         r_ipith(0)*(1d0+delta_EW), r_ipith(1)*(1d0+delta_EW), r_ipith(-1)*(1d0+delta_EW)
    print *, ""

    call Higgs_pdf_alphas_error(.false., rp_fixed, rp_onlyth, rp_onlyipi, rp_ipith)

    if (with_asmz_error) then
       print *, "Cross sections with PDF+alpha_s uncertainties (pb)"
    else
       print *, "Cross sections with PDF uncertainties (pb)"
    end if
    print format_string, "Fixed order: ", rp_fixed(0), rp_fixed(1), rp_fixed(-1)
    print format_string, "Fixed order + EW:", &
         rp_fixed(0)*(1d0+delta_EW), rp_fixed(1)*(1d0+delta_EW), rp_fixed(-1)*(1d0+delta_EW)
    print format_string, "Only threshold resummed: ", rp_onlyth(0), rp_onlyth(1), rp_onlyth(-1)
    print format_string, "Only pi^2 resummed: ", rp_onlyipi(0), rp_onlyipi(1), rp_onlyipi(-1)
    print format_string, "Threshold+pi^2 resummed: ", rp_ipith(0), rp_ipith(1), rp_ipith(-1)
    print format_string, "Threshold+pi^2 resummed + EW:", &
         rp_ipith(0)*(1d0+delta_EW), rp_ipith(1)*(1d0+delta_EW), rp_ipith(-1)*(1d0+delta_EW)
    print *, ""
  end subroutine Run_Single

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Helper subroutines

  subroutine Higgs_scale_error(onlybest, r_fixed, r_onlyth, r_onlyipi, r_ipith)
    implicit none
    logical, intent(in) :: onlybest
    real(8), intent(out) :: r_fixed(-1:1), r_onlyth(-1:1), r_onlyipi(-1:1), r_ipith(-1:1)
    real(8) :: muf00
    real(8) :: r_leading_central, r_leading_half, r_leading_double
    real(8) :: r_nonsingular_central, r_nonsingular_half, r_nonsingular_double
    real(8) :: e_fixed(2), e_onlyth(2), e_onlyipi(2), e_ipith(2)

    muf00 = muf0

    r_nonsingular_central = Higgs_Nonsingular_CS()
    r_ipith(0) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_central
    if (.not. onlybest) then
       r_leading_central = Higgs_Leading_CS()
       r_fixed(0) = r_leading_central + r_nonsingular_central
       r_onlyth(0) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular_central
       r_onlyipi(0) = Higgs_Resummed_CS(.false., .true.) + r_nonsingular_central
    end if

    muf0 = 0.5d0*muf00
    r_nonsingular_half = Higgs_Nonsingular_CS()
    e_ipith(1) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_half - r_ipith(0)
    if (.not. onlybest) then
       r_leading_half = Higgs_Leading_CS()
       e_fixed(1) = r_leading_half + r_nonsingular_half - r_fixed(0)
       e_onlyth(1) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular_half - r_onlyth(0)
       e_onlyipi(1) = Higgs_Resummed_CS(.false., .true.) + r_nonsingular_half - r_onlyipi(0)
    end if
    muf0 = 2d0*muf00
    r_nonsingular_double = Higgs_Nonsingular_CS()
    e_ipith(2) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_double - r_ipith(0)
    if (.not. onlybest) then
       r_leading_double = Higgs_Leading_CS()
       e_fixed(2) = r_leading_double + r_nonsingular_double - r_fixed(0)
       e_onlyth(2) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular_double - r_onlyth(0)
       e_onlyipi(2) = Higgs_Resummed_CS(.false., .true.) + r_nonsingular_double - r_onlyipi(0)
    end if
    muf0 = muf00
    r_ipith(1) = max(e_ipith(1), e_ipith(2), 0d0)**2
    r_ipith(-1) = min(e_ipith(1), e_ipith(2), 0d0)**2
    if (.not. onlybest) then
       r_fixed(1) = max(e_fixed(1), e_fixed(2), 0d0)
       r_fixed(-1) = min(e_fixed(1), e_fixed(2), 0d0)
       r_onlyth(1) = max(e_onlyth(1), e_onlyth(2), 0d0)**2
       r_onlyth(-1) = min(e_onlyth(1), e_onlyth(2), 0d0)**2
       r_onlyipi(1) = max(e_onlyipi(1), e_onlyipi(2), 0d0)
       r_onlyipi(-1) = min(e_onlyipi(1), e_onlyipi(2), 0d0)
    end if

    mut0 = -0.5d0
    e_ipith(1) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_central - r_ipith(0)
    if (.not. onlybest) then
       e_onlyth(1) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular_central - r_onlyth(0)
    end if
    mut0 = -2d0
    e_ipith(2) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_central - r_ipith(0)
    if (.not. onlybest) then
       e_onlyth(2) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular_central - r_onlyth(0)
    end if
    mut0 = -1d0
    r_ipith(1) = r_ipith(1) + max(e_ipith(1), e_ipith(2), 0d0)**2
    r_ipith(-1) = r_ipith(-1) + min(e_ipith(1), e_ipith(2), 0d0)**2
    if (.not. onlybest) then
       r_onlyth(1) = r_onlyth(1) + max(e_onlyth(1), e_onlyth(2), 0d0)**2
       r_onlyth(-1) = r_onlyth(-1) + min(e_onlyth(1), e_onlyth(2), 0d0)**2
    end if

    muh0 = -0.5d0
    e_ipith(1) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_central - r_ipith(0)
    if (.not. onlybest) then
       e_onlyth(1) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular_central - r_onlyth(0)
    end if
    muh0 = -2d0
    e_ipith(2) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_central - r_ipith(0)
    if (.not. onlybest) then
       e_onlyth(2) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular_central - r_onlyth(0)
    end if
    muh0 = -1d0
    r_ipith(1) = r_ipith(1) + max(e_ipith(1), e_ipith(2), 0d0)**2
    r_ipith(-1) = r_ipith(-1) + min(e_ipith(1), e_ipith(2), 0d0)**2
    if (.not. onlybest) then
       r_onlyth(1) = r_onlyth(1) + max(e_onlyth(1), e_onlyth(2), 0d0)**2
       r_onlyth(-1) = r_onlyth(-1) + min(e_onlyth(1), e_onlyth(2), 0d0)**2
    end if

    mus0 = -0d0
    e_ipith(1) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_central - r_ipith(0)
    if (.not. onlybest) then
       e_onlyth(1) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular_central - r_onlyth(0)
    end if
    mus0 = -2d0
    e_ipith(2) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_central - r_ipith(0)
    if (.not. onlybest) then
       e_onlyth(2) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular_central - r_onlyth(0)
    end if
    mus0 = -1d0

    r_ipith(1) = r_ipith(1) + max(e_ipith(1), e_ipith(2), 0d0)**2
    r_ipith(-1) = r_ipith(-1) + min(e_ipith(1), e_ipith(2), 0d0)**2
    r_ipith(1) = sqrt(r_ipith(1))
    r_ipith(-1) = -sqrt(r_ipith(-1))
    if (.not. onlybest) then
       r_onlyth(1) = r_onlyth(1) + max(e_onlyth(1), e_onlyth(2), 0d0)**2
       r_onlyth(-1) = r_onlyth(-1) + min(e_onlyth(1), e_onlyth(2), 0d0)**2
       r_onlyth(1) = sqrt(r_onlyth(1))
       r_onlyth(-1) = -sqrt(r_onlyth(-1))
    end if
  end subroutine Higgs_scale_error

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  
  subroutine Higgs_pdf_alphas_error(onlybest, r_fixed, r_onlyth, r_onlyipi, r_ipith)
    implicit none
    logical, intent(in) :: onlybest
    real(8), intent(out) :: r_fixed(-1:1), r_onlyth(-1:1), r_onlyipi(-1:1), r_ipith(-1:1)
    if (pdfset(1:4) == "MSTW") then
       call Higgs_pdf_alphas_error_MSTW(onlybest, r_fixed, r_onlyth, r_onlyipi, r_ipith)
    else if (pdfset(1:5) == "NNPDF") then
       call Higgs_pdf_alphas_error_NNPDF(onlybest, r_fixed, r_onlyth, r_onlyipi, r_ipith)
    else
       call Higgs_pdf_alphas_error_CTEQ(onlybest, r_fixed, r_onlyth, r_onlyipi, r_ipith)
    end if
  end subroutine Higgs_pdf_alphas_error

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Compute the PDF+alpha_s uncertainties according to arXiv:0905.3531
  subroutine Higgs_pdf_alphas_error_MSTW(onlybest, r_fixed, r_onlyth, r_onlyipi, r_ipith)
    implicit none
    logical, intent(in) :: onlybest
    real(8), intent(out) :: r_fixed(-1:1), r_onlyth(-1:1), r_onlyipi(-1:1), r_ipith(-1:1)
    real(8), allocatable :: r_leading(:,:), r_nonsingular(:,:)
    real(8), allocatable :: r_fixed_data(:,:), r_onlyth_data(:,:), r_onlyipi_data(:,:), r_ipith_data(:,:)
    real(8), allocatable :: e_fixed(:,:), e_onlyth(:,:), e_onlyipi(:,:), e_ipith(:,:)
    real(8) :: asmz0
    integer :: i, j, n

    if (with_asmz_error) then
       n = n_asmz
    else
       n = 0
    end if

    allocate ( r_nonsingular(0:n,0:nmem) )
    allocate ( r_ipith_data(0:n,0:nmem) )
    allocate ( e_ipith(2,0:n) )
    if (.not. onlybest) then
       allocate ( r_leading(0:n,0:nmem) )
       allocate ( r_fixed_data(0:n,0:nmem), r_onlyth_data(0:n,0:nmem), r_onlyipi_data(0:n,0:nmem) )
       allocate ( e_fixed(2,0:n), e_onlyth(2,0:n), e_onlyipi(2,0:n) )
    end if

    do j = 0, nmem
       call InitPDF(j)
       r_nonsingular(0,j) = Higgs_Nonsingular_CS()
       r_ipith_data(0,j) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular(0,j)
       if (.not. onlybest) then
          r_leading(0,j) = Higgs_Leading_CS()
          r_fixed_data(0,j) = r_leading(0,j) + r_nonsingular(0,j)
          r_onlyth_data(0,j) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular(0,j)
          r_onlyipi_data(0,j) = Higgs_Resummed_CS(.false., .true.) + r_nonsingular(0,j)
       end if
    end do

    if (with_asmz_error) then
       asmz0 = alphas_mZ
       do i = 1, n
          alphas_mZ = asmz_asmz(i)
          call InitQCD
          call InitPDFSetByName(pdfset_asmz(i))
          do j = 0, nmem
             call InitPDF(j)
             r_nonsingular(i,j) = Higgs_Nonsingular_CS()
             r_ipith_data(i,j) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular(i,j)
             if (.not. onlybest) then
                r_leading(i,j) = Higgs_Leading_CS()
                r_fixed_data(i,j) = r_leading(i,j) + r_nonsingular(i,j)
                r_onlyth_data(i,j) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular(i,j)
                r_onlyipi_data(i,j) = Higgs_Resummed_CS(.false., .true.) + r_nonsingular(i,j)
             end if
          end do
       end do
       alphas_mZ = asmz0
       call InitQCD
    end if
    
    r_ipith(0) = r_ipith_data(0,0)
    e_ipith = 0d0
    if (.not. onlybest) then
       r_fixed(0) = r_fixed_data(0,0)
       r_onlyth(0) = r_onlyth_data(0,0)
       r_onlyipi(0) = r_onlyipi_data(0,0)
       e_fixed = 0d0
       e_onlyth = 0d0
       e_onlyipi = 0d0
    end if
    do i = 0, n
       do j = 0, nmem/2-1
          e_ipith(1,i) = e_ipith(1,i) &
               + max(r_ipith_data(i,2*j+1)-r_ipith_data(i,0), r_ipith_data(i,2*j+2)-r_ipith_data(i,0), 0d0)**2
          e_ipith(2,i) = e_ipith(2,i) &
               + min(r_ipith_data(i,2*j+1)-r_ipith_data(i,0), r_ipith_data(i,2*j+2)-r_ipith_data(i,0), 0d0)**2
          if (.not. onlybest) then
             e_fixed(1,i) = e_fixed(1,i) &
                  + max(r_fixed_data(i,2*j+1)-r_fixed_data(i,0), r_fixed_data(i,2*j+2)-r_fixed_data(i,0), 0d0)**2
             e_fixed(2,i) = e_fixed(2,i) &
                  + min(r_fixed_data(i,2*j+1)-r_fixed_data(i,0), r_fixed_data(i,2*j+2)-r_fixed_data(i,0), 0d0)**2
             e_onlyth(1,i) = e_onlyth(1,i) &
                  + max(r_onlyth_data(i,2*j+1)-r_onlyth_data(i,0), r_onlyth_data(i,2*j+2)-r_onlyth_data(i,0), 0d0)**2
             e_onlyth(2,i) = e_onlyth(2,i) &
                  + min(r_onlyth_data(i,2*j+1)-r_onlyth_data(i,0), r_onlyth_data(i,2*j+2)-r_onlyth_data(i,0), 0d0)**2
             e_onlyipi(1,i) = e_onlyipi(1,i) &
                  + max(r_onlyipi_data(i,2*j+1)-r_onlyipi_data(i,0), r_onlyipi_data(i,2*j+2)-r_onlyipi_data(i,0), 0d0)**2
             e_onlyipi(2,i) = e_onlyipi(2,i) &
                  + min(r_onlyipi_data(i,2*j+1)-r_onlyipi_data(i,0), r_onlyipi_data(i,2*j+2)-r_onlyipi_data(i,0), 0d0)**2
          end if
       end do
       e_ipith(1,i) = r_ipith_data(i,0) + sqrt(e_ipith(1,i))
       e_ipith(2,i) = r_ipith_data(i,0) - sqrt(e_ipith(2,i))
       if (.not. onlybest) then
          e_fixed(1,i) = r_fixed_data(i,0) + sqrt(e_fixed(1,i))
          e_fixed(2,i) = r_fixed_data(i,0) - sqrt(e_fixed(2,i))
          e_onlyth(1,i) = r_onlyth_data(i,0) + sqrt(e_onlyth(1,i))
          e_onlyth(2,i) = r_onlyth_data(i,0) - sqrt(e_onlyth(2,i))
          e_onlyipi(1,i) = r_onlyipi_data(i,0) + sqrt(e_onlyipi(1,i))
          e_onlyipi(2,i) = r_onlyipi_data(i,0) - sqrt(e_onlyipi(2,i))
       end if
    end do
    
    r_ipith(1) = maxval(e_ipith(1,:)) - r_ipith(0)
    r_ipith(-1) = minval(e_ipith(2,:)) - r_ipith(0)
    if (.not. onlybest) then
       r_fixed(1) = maxval(e_fixed(1,:)) - r_fixed(0)
       r_fixed(-1) = minval(e_fixed(2,:)) - r_fixed(0)
       r_onlyth(1) = maxval(e_onlyth(1,:)) - r_onlyth(0)
       r_onlyth(-1) = minval(e_onlyth(2,:)) - r_onlyth(0)
       r_onlyipi(1) = maxval(e_onlyipi(1,:)) - r_onlyipi(0)
       r_onlyipi(-1) = minval(e_onlyipi(2,:)) - r_onlyipi(0)
    end if

    deallocate ( r_nonsingular )
    deallocate ( r_ipith_data )
    deallocate ( e_ipith )
    if (.not. onlybest) then
       deallocate ( r_leading )
       deallocate ( r_fixed_data, r_onlyth_data, r_onlyipi_data )
       deallocate ( e_fixed, e_onlyth, e_onlyipi )
    end if

    call InitPDFSetByName(pdfset)
    call InitPDF(0)
  end subroutine Higgs_pdf_alphas_error_MSTW

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Compute the PDF+alpha_s uncertainties using the "add in quadrature" method,
! which is a good approximation according to arXiv:1004.0962
  subroutine Higgs_pdf_alphas_error_NNPDF(onlybest, r_fixed, r_onlyth, r_onlyipi, r_ipith)
    implicit none
    logical, intent(in) :: onlybest
    real(8), intent(out) :: r_fixed(-1:1), r_onlyth(-1:1), r_onlyipi(-1:1), r_ipith(-1:1)
    real(8), allocatable :: r_leading(:), r_nonsingular(:)
    real(8), allocatable :: r_fixed_data(:), r_onlyth_data(:), r_onlyipi_data(:), r_ipith_data(:)
    !!!!!!!!!! assuming n_asmz = 2 !!!!!!!!!!
    real(8) :: r_leading_asmz(2), r_nonsingular_asmz(2)
    real(8) :: r_fixed_asmz(2), r_onlyth_asmz(2), r_onlyipi_asmz(2), r_ipith_asmz(2)
    real(8) :: asmz0
    integer :: i

    allocate ( r_nonsingular(nmem) )
    allocate ( r_ipith_data(nmem) )
    if (.not. onlybest) then
       allocate ( r_leading(nmem) )
       allocate ( r_fixed_data(nmem), r_onlyth_data(nmem), r_onlyipi_data(nmem) )
    end if

    do i = 1, nmem
       call InitPDF(i)
       r_nonsingular(i) = Higgs_Nonsingular_CS()
       r_ipith_data(i) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular(i)
       if (.not. onlybest) then
          r_leading(i) = Higgs_Leading_CS()
          r_fixed_data(i) = r_leading(i) + r_nonsingular(i)
          r_onlyth_data(i) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular(i)
          r_onlyipi_data(i) = Higgs_Resummed_CS(.false., .true.) + r_nonsingular(i)
       end if
    end do

    r_ipith = 0d0
    if (.not. onlybest) then
       r_fixed = 0d0
       r_onlyth = 0d0
       r_onlyipi = 0d0
    end if
    do i = 1, nmem
       r_ipith(0) = r_ipith(0) + r_ipith_data(i)/nmem
       if (.not. onlybest) then
          r_fixed(0) = r_fixed(0) + r_fixed_data(i)/nmem
          r_onlyth(0) = r_onlyth(0) + r_onlyth_data(i)/nmem
          r_onlyipi(0) = r_onlyipi(0) + r_onlyipi_data(i)/nmem
       end if
    end do
    do i = 1, nmem
       r_ipith(1) = r_ipith(1) + (r_ipith_data(i)-r_ipith(0))**2/(nmem-1)
       if (.not. onlybest) then
          r_fixed(1) = r_fixed(1) + (r_fixed_data(i)-r_fixed(0))**2/(nmem-1)
          r_onlyth(1) = r_onlyth(1) + (r_onlyth_data(i)-r_onlyth(0))**2/(nmem-1)
          r_onlyipi(1) = r_onlyipi(1) + (r_onlyipi_data(i)-r_onlyipi(0))**2/(nmem-1)
       end if
    end do

    if (with_asmz_error) then
       asmz0 = alphas_mZ
       do i = 1, 2
          alphas_mZ = asmz_asmz(i)
          call InitQCD
          call InitPDFSetByName(pdfset_asmz(i))
          call InitPDF(0)
          r_nonsingular_asmz(i) = Higgs_Nonsingular_CS()
          r_ipith_asmz(i) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular_asmz(i)
          if (.not. onlybest) then
             r_leading_asmz(i) = Higgs_Leading_CS()
             r_fixed_asmz(i) = r_leading_asmz(i) + r_nonsingular_asmz(i)
             r_onlyth_asmz(i) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular_asmz(i)
             r_onlyipi_asmz(i) = Higgs_Resummed_CS(.false., .true.) + r_nonsingular_asmz(i)
          end if
       end do
       r_ipith(1) = r_ipith(1) + ((r_ipith_asmz(1)-r_ipith_asmz(2))/2d0)**2
       if (.not. onlybest) then
          r_fixed(1) = r_fixed(1) + ((r_fixed_asmz(1)-r_fixed_asmz(2))/2d0)**2
          r_onlyth(1) = r_onlyth(1) + ((r_onlyth_asmz(1)-r_onlyth_asmz(2))/2d0)**2
          r_onlyipi(1) = r_onlyipi(1) + ((r_onlyipi_asmz(1)-r_onlyipi_asmz(2))/2d0)**2
       end if
       alphas_mZ = asmz0
       call InitQCD
    end if

    r_ipith(1) = sqrt(r_ipith(1)) * 1.64485d0
    r_ipith(-1) = -r_ipith(1)
    if (.not. onlybest) then
       r_fixed(1) = sqrt(r_fixed(1)) * 1.64485d0
       r_fixed(-1) = -r_fixed(1)
       r_onlyth(1) = sqrt(r_onlyth(1)) * 1.64485d0
       r_onlyth(-1) = -r_onlyth(1)
       r_onlyipi(1) = sqrt(r_onlyipi(1)) * 1.64485d0
       r_onlyipi(-1) = -r_onlyipi(1)
    end if

    deallocate ( r_nonsingular )
    deallocate ( r_ipith_data )
    if (.not. onlybest) then
       deallocate ( r_leading )
       deallocate ( r_fixed_data, r_onlyth_data, r_onlyipi_data )
    end if

    call InitPDFSetByName(pdfset)
    call InitPDF(0)
  end subroutine Higgs_pdf_alphas_error_NNPDF

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! Compute the PDF+alpha_s uncertainties according to arXiv:1004.4624
  subroutine Higgs_pdf_alphas_error_CTEQ(onlybest, r_fixed, r_onlyth, r_onlyipi, r_ipith)
    implicit none
    logical, intent(in) :: onlybest
    real(8), intent(out) :: r_fixed(-1:1), r_onlyth(-1:1), r_onlyipi(-1:1), r_ipith(-1:1)
    real(8) :: r_leading, r_nonsingular
    real(8) :: e_fixed(2), e_onlyth(2), e_onlyipi(2), e_ipith(2)
    real(8) :: asmz0
    integer :: i, j

    r_ipith = 0d0
    r_nonsingular = Higgs_Nonsingular_CS()
    r_ipith(0) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular
    if (.not. onlybest) then
       r_fixed = 0d0
       r_onlyth = 0d0
       r_onlyipi = 0d0
       r_leading = Higgs_Leading_CS()
       r_fixed(0) = r_leading + r_nonsingular
       r_onlyth(0) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular
       r_onlyipi(0) = Higgs_Resummed_CS(.false., .true.) + r_nonsingular
    end if

    do i = 0, nmem/2-1
       do j = 1, 2
          call InitPDF(i*2+j)
          r_nonsingular = Higgs_Nonsingular_CS()
          e_ipith(j) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular - r_ipith(0)
          if (.not. onlybest) then
             r_leading = Higgs_Leading_CS()
             e_fixed(j) = r_leading + r_nonsingular - r_fixed(0)
             e_onlyth(j) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular - r_onlyth(0)
             e_onlyipi(j) = Higgs_Resummed_CS(.false., .true.) + r_nonsingular - r_onlyipi(0)
          end if
       enddo
       r_ipith(1) = r_ipith(1) + max(e_ipith(1), e_ipith(2), 0d0)**2
       r_ipith(-1) = r_ipith(-1) + min(e_ipith(1), e_ipith(2), 0d0)**2
       if (.not. onlybest) then
          r_fixed(1) = r_fixed(1) + max(e_fixed(1), e_fixed(2), 0d0)**2
          r_fixed(-1) = r_fixed(-1) + min(e_fixed(1), e_fixed(2), 0d0)**2
          r_onlyth(1) = r_onlyth(1) + max(e_onlyth(1), e_onlyth(2), 0d0)**2
          r_onlyth(-1) = r_onlyth(-1) + min(e_onlyth(1), e_onlyth(2), 0d0)**2
          r_onlyipi(1) = r_onlyipi(1) + max(e_onlyipi(1), e_onlyipi(2), 0d0)**2
          r_onlyipi(-1) = r_onlyipi(-1) + min(e_onlyipi(1), e_onlyipi(2), 0d0)**2
       end if
    end do

    if (with_asmz_error) then
       asmz0 = alphas_mZ
       call InitPDFSetByName(pdfset_asmz(1))
       call numberPDF(nmem)
       ! assuming n_asmz = 2, and iset_\pm = iset_central \pm 2
       do i = 1, 2
          alphas_mZ = asmz_asmz(i)
          call InitQCD
          call InitPDF(nmem/2+(-1)**i*2)
          r_nonsingular = Higgs_Nonsingular_CS()
          e_ipith(i) = Higgs_Resummed_CS(.true., .true.) + r_nonsingular - r_ipith(0)
          if (.not. onlybest) then
             r_leading = Higgs_Leading_CS()
             e_fixed(i) = r_leading + r_nonsingular - r_fixed(0)
             e_onlyth(i) = Higgs_Resummed_CS(.true., .false.) + r_nonsingular - r_onlyth(0)
             e_onlyipi(i) = Higgs_Resummed_CS(.false., .true.) + r_nonsingular - r_onlyipi(0)
          end if
       end do
       r_ipith(1) = r_ipith(1) + max(e_ipith(1), e_ipith(2), 0d0)**2
       r_ipith(-1) = r_ipith(-1) + min(e_ipith(1), e_ipith(2), 0d0)**2
       if (.not. onlybest) then
          r_fixed(1) = r_fixed(1) + max(e_fixed(1), e_fixed(2), 0d0)**2
          r_fixed(-1) = r_fixed(-1) + min(e_fixed(1), e_fixed(2), 0d0)**2
          r_onlyth(1) = r_onlyth(1) + max(e_onlyth(1), e_onlyth(2), 0d0)**2
          r_onlyth(-1) = r_onlyth(-1) + min(e_onlyth(1), e_onlyth(2), 0d0)**2
          r_onlyipi(1) = r_onlyipi(1) + max(e_onlyipi(1), e_onlyipi(2), 0d0)**2
          r_onlyipi(-1) = r_onlyipi(-1) + min(e_onlyipi(1), e_onlyipi(2), 0d0)**2
       end if
       alphas_mZ = asmz0
       call InitQCD
    end if

    r_ipith(1) = sqrt(r_ipith(1))
    r_ipith(-1) = -sqrt(r_ipith(-1))
    if (.not. onlybest) then
       r_fixed(1) = sqrt(r_fixed(1))
       r_fixed(-1) = -sqrt(r_fixed(-1))
       r_onlyth(1) = sqrt(r_onlyth(1))
       r_onlyth(-1) = -sqrt(r_onlyth(-1))
       r_onlyipi(1) = sqrt(r_onlyipi(1))
       r_onlyipi(-1) = -sqrt(r_onlyipi(-1))
    end if

    call InitPDFSetByName(pdfset)
    call numberPDF(nmem)
    call InitPDF(0)
  end subroutine Higgs_pdf_alphas_error_CTEQ

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  subroutine Run_mH
    implicit none
    real(8) :: r_ipith(-1:1), rp_ipith(-1:1), dummy(-1:1)
    integer :: i
    do i = 0, 17
       mH = 115d0 + i*5d0
       call Higgs_scale_error(.true., dummy, dummy, dummy, r_ipith)
       call Higgs_pdf_alphas_error(.true., dummy, dummy, dummy, rp_ipith)
       print *, mH, r_ipith(0)*(1d0+delta_EW), r_ipith(1)*(1d0+delta_EW), r_ipith(-1)*(1d0+delta_EW), &
            rp_ipith(1)*(1d0+delta_EW), rp_ipith(-1)*(1d0+delta_EW)
    end do
  end subroutine Run_mH

  subroutine Run_mH_Band
    implicit none
    real(8) :: r_ipith(-1:1), rp_ipith(-1:1), dummy(-1:1)
    integer :: i
    iERR = 5d-3
    do i = 0, 100
       mH = 100d0 + i
       call Higgs_scale_error(.true., dummy, dummy, dummy, r_ipith)
       call Higgs_pdf_alphas_error(.true., dummy, dummy, dummy, rp_ipith)
       print *, mH, r_ipith(0)*(1d0+delta_EW), r_ipith(1)*(1d0+delta_EW), r_ipith(-1)*(1d0+delta_EW), &
            rp_ipith(1)*(1d0+delta_EW), rp_ipith(-1)*(1d0+delta_EW)
    end do
  end subroutine Run_mH_Band

  subroutine Run_Ecms
    implicit none
    real(8) :: r_ipith(-1:1), rp_ipith(-1:1), dummy(-1:1)
    integer :: i
    iERR = 5d-3
    do i = 0, 35
       Ecms = 7000d0 + i*200d0
       call Higgs_scale_error(.true., dummy, dummy, dummy, r_ipith)
       call Higgs_pdf_alphas_error(.true., dummy, dummy, dummy, rp_ipith)
       print *, Ecms, r_ipith(0)*(1d0+delta_EW), r_ipith(1)*(1d0+delta_EW), r_ipith(-1)*(1d0+delta_EW), &
            rp_ipith(1)*(1d0+delta_EW), rp_ipith(-1)*(1d0+delta_EW)
    end do
  end subroutine Run_Ecms

end program RunHiggs
